%
%    This file is part of the INTERACT Toolbox
%    Copyright 2008 Martin Vejmelka
% 
%	 File: estimate_wphdiff_entropy.m
%     Version: 1.0
%	 Date: 14 April 2008
%     License: GPL version 3 or later
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%  
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%  
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    Author: Martin Vejmelka (bug reports: vejmelka at cs dot cas.cz)
%
% Description:
%
% 	Computes the standard binned (histogram-based) entropy
%	from a finite sample.  All bins are of equal size.  Does
%	not perform any bias corrections. The function works with 
%	wrapped phase differences.
%
% Synopsis:
%
%	ent = estimate_wphdiff_entropy(wph_diff, bins)
%
% Arguments:
%
%   wph_diff: the wrapped phase differences, must be a vector
%
%   bins: the number of bins into which the series must be split
%
% Returns:
%
%   ent: the entropy of the wrapped phase differences
%
% Examples:
%
%	None.
%
function ent = estimate_wphdiff_entropy(wph_diff, bins)

	% PRE: check inputs
	if(size(wph_diff,1) > 1 && size(wph_diff,2) > 1)
		error('Input must be a vector.');
	end

	N = length(wph_diff);

	% prepare 'bins', fill with 0
	pdf = zeros(1,bins);

	% Compute the index of the bin to which each sample
	% belongs.
	bin_width = 2*pi / bins;
	bin_ndx = floor(wph_diff / bin_width) + 1;

	% Accumulate samples into their bins to obtain the number of samples
	% in each bin
	for i=1:N, pdf(bin_ndx(i)) = pdf(bin_ndx(i)) + 1; end
	
	% Normalize bins to obtain probabilities
	pdf = pdf / N;

	% Compute the entropy using the formula ent = sum of prob_i * log(prob_i)
	% The if statement implements the agreement: 0 * log(0) = 0
	ent = 0;
	for i=1:bins, if(pdf(i) > 0), ent = ent - pdf(i) * log(pdf(i)); end, end;

	% normalize the entropy to [0...1]
	ent = (log(bins) - ent) / log(bins);
